/*********************************************************************NVMH3****
Copyright NVIDIA Corporation 2003
TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
*AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.


Comments:
	Simple ocean shader with animated bump map and geometric waves
	Based partly on "Effective Water Simulation From Physical Models", GPU Gems

11 Aug 05: converted from HLSL to GLSL by Jeff Doyle (nfz) to work in Ogre

10 Feb 09: - added sun glow effect based on the shaders by Byungil Jeong
           (http://www.evl.uic.edu/bijeong/cs594/proj2/)
           - removed the refraction map code and modified to work with FlightGear
           -- Benoît Laniel

******************************************************************************/

uniform sampler2D NormalMap;
uniform sampler2DRect EnvironmentMap;
uniform vec4 sunColor;
uniform vec2 winSize;
uniform bool hq;

varying vec2 bumpCoord0;
varying vec2 bumpCoord1;
varying vec2 bumpCoord2;
varying vec3 eyeDir;
varying vec3 lightVector;
varying vec3 eyeVector;

void main(void)
{
  const vec4 deepColor = vec4(0.0, 0.3, 0.5, 1.0);
  const vec4 shallowColor = vec4(0.05, 0.08, 0.16, 1.0);
  const float reflectionAmount = 1.0;
  const float waterAmount = 0.3;
  const float fresnelPower = 5.0;
  const float fresnelBias = 0.328;
  const float hdrMultiplier = 0.471;
  const float shininess = 64.0;

  // sum normal maps
  // sample from 3 different points so no texture repetition is noticeable
  vec4 t0 = texture2D(NormalMap, bumpCoord0) * 2.0 - 1.0;
  vec4 t1 = texture2D(NormalMap, bumpCoord1) * 2.0 - 1.0;
  vec4 t2 = texture2D(NormalMap, bumpCoord2) * 2.0 - 1.0;
  vec3 N = t0.xyz + t1.xyz + t2.xyz;
  N = normalize(N);

  // if hq use envmap, else use only a color
  vec4 reflection = vec4(0.35, 0.46, 0.72, 1.0);
  if (hq)
  {
    vec2 texCoord = gl_FragCoord.xy + N.xy * winSize;
    reflection = texture2DRect(EnvironmentMap, texCoord);
  }

  // cheap hdr effect
  reflection.rgb *= (reflection.r + reflection.g + reflection.b) * hdrMultiplier;

  // modulate reflection and sun color
  reflection = vec4(reflection.rgb * sunColor.rgb, reflection.a * sunColor.a);

  // fresnel
  vec3 E = normalize(eyeDir);
  float facing = 1.0 - dot(-E, N);
  float fresnel = clamp(fresnelBias + pow(facing, fresnelPower), 0.0, 1.0);
  vec4 waterColor = mix(shallowColor, deepColor, facing) * waterAmount;
  reflection = mix(waterColor, reflection, fresnel) * reflectionAmount;

  // glow effect
  vec3 E2 = normalize(eyeVector);
  vec3 L = normalize(reflect(lightVector, N));
  float intensity = max(0.0, dot(L, -E2));

  // water + reflection + glow
  vec4 final_color = waterColor + reflection + sunColor*pow(intensity, shininess);

  // add fog
  const float LOG2E = 1.442695; // = 1 / log( 2 )
  float fog = exp2(-gl_Fog.density * gl_FogFragCoord * LOG2E);
  fog = clamp(fog, 0.0, 1.0);
  gl_FragColor = vec4(mix(vec3(gl_Fog.color), vec3(final_color), fog), final_color.a);
}
