/*********************************************************************NVMH3****
Copyright NVIDIA Corporation 2003
TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
*AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.


Comments:
	Simple ocean shader with animated bump map and geometric waves
	Based partly on "Effective Water Simulation From Physical Models", GPU Gems

11 Aug 05: converted from HLSL to GLSL by Jeff Doyle (nfz) to work in Ogre

10 Feb 09: - added sun glow effect based on the shaders by Byungil Jeong
           (http://www.evl.uic.edu/bijeong/cs594/proj2/)
           - removed the refraction map code and modified to work with FlightGear
           -- Benoît Laniel

******************************************************************************/

uniform vec3 eyePosition;
uniform vec3 lightPosition;
uniform float time;

varying vec2 bumpCoord0;
varying vec2 bumpCoord1;
varying vec2 bumpCoord2;
varying vec3 eyeDir;
varying vec3 lightVector;
varying vec3 eyeVector;

// wave functions
struct Wave {
  float freq;  // 2*PI / wavelength
  float amp;   // amplitude
  float phase; // speed * 2*PI / wavelength
  vec2 dir;
};

void main(void)
{
  const vec2 textureScale = vec2(1.0, 1.0);
  const vec2 bumpSpeed = vec2(0.015, 0.005);
  const bool move_geometry = false;

  vec4 P = gl_Vertex;

  // moving geometry
  if (move_geometry)
  {
    const float waveFreq = 0.028;
    const float waveAmp = 1.8;

    #define NWAVES 2
    Wave wave[NWAVES];
    wave[0] = Wave(waveFreq, waveAmp, 0.5, vec2(-1.0, 0.0));
    wave[1] = Wave(3.0 * waveFreq, 0.33 * waveAmp, 1.7, vec2(-0.7, 0.7));

    // sum waves
    float ddx = 0.0, ddy = 0.0;
    float deriv;
    float angle;

    // wave synthesis using two sine waves at different frequencies and phase shift
    for(int i = 0; i < NWAVES; ++i)
    {
      angle = dot(wave[i].dir, P.xz) * wave[i].freq + time * wave[i].phase;
      P.y += wave[i].amp * sin( angle );
      // calculate derivate of wave function
      deriv = wave[i].freq * wave[i].amp * cos(angle);
      ddx -= deriv * wave[i].dir.x;
      ddy -= deriv * wave[i].dir.y;
    }

    gl_Position = gl_ModelViewProjectionMatrix * P;
  }
  else
  {
    gl_Position = ftransform();
  }

  // calculate texture coordinates for normal map lookup
  bumpCoord0.xy = gl_MultiTexCoord0.xy * textureScale + time * bumpSpeed;
  bumpCoord1.xy = gl_MultiTexCoord0.xy * textureScale * 2.0 + time * bumpSpeed * 4.0;
  bumpCoord2.xy = gl_MultiTexCoord0.xy * textureScale * 4.0 + time * bumpSpeed * 8.0;

  eyeDir = P.xyz - eyePosition;

  // glow effect

  // automatic generation of tangential and binomal vec
  // from normal vector
  vec3 vTangent;
  if (gl_Normal.x == 0.0 && gl_Normal.z == 0.0)
    vTangent = vec3(1.0, 0.0, 0.0);
  else
    vTangent = vec3(gl_Normal.z, 0, -gl_Normal.x);

  // transform to eye coordinates
  vec3 n = normalize(gl_NormalMatrix * gl_Normal);
  vec3 t = normalize(gl_NormalMatrix * vTangent);
  vec3 b = cross(n, t);

  // vertex in eye coordinate
  vec4 vVertex = gl_ModelViewMatrix * P;

  // light direction in eye coordinate
  vec3 tmpVec = vec3(gl_ModelViewMatrix * vec4(lightPosition, 1.0) - vVertex);

  // transform light vector into tangent space
  lightVector.x = dot(tmpVec, t);
  lightVector.y = dot(tmpVec, b);
  lightVector.z = dot(tmpVec, n);

  lightVector = normalize(lightVector);

  // transform view vector into tangent space
  tmpVec = -vVertex.xyz;
  eyeVector.x = dot(tmpVec, t);
  eyeVector.y = dot(tmpVec, b);
  eyeVector.z = dot(tmpVec, n);

  // fog
  gl_FogFragCoord = abs(vVertex.z);
}
